@@ -36,6 +36,7 @@
 
 #include "Configuration.hpp"
 #include "Utils.h"
+#include <LoggingAgent/FilterSupport.h>
 
 /* The APR headers must come after the Passenger headers. See Hooks.cpp
  * to learn why.
@@ -159,6 +160,17 @@ destroy_config_struct(void *x) {
 	return APR_SUCCESS;
 }
 
+template<typename Collection, typename T> static bool
+contains(const Collection &coll, const T &item) {
+	typename Collection::const_iterator it;
+	for (it = coll.begin(); it != coll.end(); it++) {
+		if (*it == item) {
+			return true;
+		}
+	}
+	return false;
+}
+
 
 extern "C" {
 
@@ -176,8 +188,7 @@ passenger_config_create_dir(apr_pool_t *p, char *dirspec) {
 	config->autoDetectRails = DirConfig::UNSET;
 	config->autoDetectRack = DirConfig::UNSET;
 	config->autoDetectWSGI = DirConfig::UNSET;
-	config->railsEnv = NULL;
-	config->rackEnv = NULL;
+	config->environment = NULL;
 	config->appRoot = NULL;
 	config->user = NULL;
 	config->group = NULL;
@@ -197,7 +208,7 @@ passenger_config_create_dir(apr_pool_t *p, char *dirspec) {
 	config->restartDir = NULL;
 	config->uploadBufferDir = NULL;
 	config->friendlyErrorPages = DirConfig::UNSET;
-	config->analytics = DirConfig::UNSET;
+	config->unionStationSupport = DirConfig::UNSET;
 	/*************************************/
 	return config;
 }
@@ -222,8 +233,7 @@ passenger_config_merge_dir(apr_pool_t *p, void *basev, void *addv) {
 	MERGE_THREEWAY_CONFIG(autoDetectRails);
 	MERGE_THREEWAY_CONFIG(autoDetectRack);
 	MERGE_THREEWAY_CONFIG(autoDetectWSGI);
-	MERGE_STR_CONFIG(railsEnv);
-	MERGE_STR_CONFIG(rackEnv);
+	MERGE_STR_CONFIG(environment);
 	MERGE_STR_CONFIG(appRoot);
 	MERGE_STRING_CONFIG(appGroupName);
 	MERGE_STR_CONFIG(user);
@@ -238,10 +248,17 @@ passenger_config_merge_dir(apr_pool_t *p, void *basev, void *addv) {
 	MERGE_INT_CONFIG(statThrottleRate);
 	MERGE_STR_CONFIG(restartDir);
 	MERGE_STR_CONFIG(uploadBufferDir);
+	MERGE_STRING_CONFIG(unionStationKey);
+	config->unionStationFilters = base->unionStationFilters;
+	for (vector<string>::const_iterator it = add->unionStationFilters.begin(); it != add->unionStationFilters.end(); it++) {
+		if (!contains(config->unionStationFilters, *it)) {
+			config->unionStationFilters.push_back(*it);
+		}
+	}
 	MERGE_THREEWAY_CONFIG(resolveSymlinksInDocRoot);
 	MERGE_THREEWAY_CONFIG(allowEncodedSlashes);
 	MERGE_THREEWAY_CONFIG(friendlyErrorPages);
-	MERGE_THREEWAY_CONFIG(analytics);
+	MERGE_THREEWAY_CONFIG(unionStationSupport);
 	/*************************************/
 	return config;
 }
@@ -262,6 +279,9 @@ DEFINE_SERVER_BOOLEAN_CONFIG_SETTER(cmd_passenger_user_switching, userSwitching)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_default_user, defaultUser)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_default_group, defaultGroup)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_temp_dir, tempDir)
+DEFINE_SERVER_STR_CONFIG_SETTER(cmd_union_station_gateway_address, unionStationGatewayAddress)
+DEFINE_SERVER_INT_CONFIG_SETTER(cmd_union_station_gateway_port, unionStationGatewayPort, int, 1)
+DEFINE_SERVER_STR_CONFIG_SETTER(cmd_union_station_gateway_cert, unionStationGatewayCert)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_analytics_log_dir, analyticsLogDir)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_analytics_log_user, analyticsLogUser)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_analytics_log_group, analyticsLogGroup)
@@ -278,6 +298,7 @@ DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_passenger_use_global_queue, useGlobalQueue
 DEFINE_DIR_INT_CONFIG_SETTER(cmd_passenger_max_requests, maxRequests, unsigned long, 0)
 DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_passenger_high_performance, highPerformance)
 DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_passenger_enabled, enabled)
+DEFINE_DIR_STR_CONFIG_SETTER(cmd_environment, environment)
 DEFINE_DIR_INT_CONFIG_SETTER(cmd_passenger_stat_throttle_rate, statThrottleRate, unsigned long, 0)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_app_root, appRoot)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_app_group_name, appGroupName)
@@ -285,10 +306,11 @@ DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_user, user)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_group, group)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_restart_dir, restartDir)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_upload_buffer_dir, uploadBufferDir)
+DEFINE_DIR_STR_CONFIG_SETTER(cmd_union_station_key, unionStationKey)
 DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_passenger_resolve_symlinks_in_document_root, resolveSymlinksInDocRoot)
 DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_passenger_allow_encoded_slashes, allowEncodedSlashes)
 DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_passenger_friendly_error_pages, friendlyErrorPages)
-DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_passenger_analytics, analytics)
+DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_union_station_support, unionStationSupport)
 
 static const char *
 cmd_passenger_spawn_method(cmd_parms *cmd, void *pcfg, const char *arg) {
@@ -305,6 +327,24 @@ cmd_passenger_spawn_method(cmd_parms *cmd, void *pcfg, const char *arg) {
 	return NULL;
 }
 
+static const char *
+cmd_union_station_filter(cmd_parms *cmd, void *pcfg, const char *arg) {
+	DirConfig *config = (DirConfig *) pcfg;
+	if (strlen(arg) == 0) {
+		return "UnionStationFilter may not be set to the empty string";
+	} else {
+		try {
+			FilterSupport::Filter f(arg);
+			config->unionStationFilters.push_back(arg);
+			return NULL;
+		} catch (const SyntaxError &e) {
+			string message = "Syntax error in Union Station filter: ";
+			message.append(e.what());
+			return strdup(message.c_str());
+		}
+	}
+}
+
 
 /*************************************************
  * Rails-specific settings
@@ -326,7 +366,6 @@ cmd_rails_base_uri(cmd_parms *cmd, void *pcfg, const char *arg) {
 }
 
 DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_rails_auto_detect, autoDetectRails)
-DEFINE_DIR_STR_CONFIG_SETTER(cmd_rails_env, railsEnv)
 
 static const char *
 cmd_rails_framework_spawner_idle_time(cmd_parms *cmd, void *pcfg, const char *arg) {
@@ -383,7 +422,6 @@ cmd_rack_base_uri(cmd_parms *cmd, void *pcfg, const char *arg) {
 }
 
 DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_rack_auto_detect, autoDetectRack)
-DEFINE_DIR_STR_CONFIG_SETTER(cmd_rack_env, rackEnv)
 
 
 /*************************************************
@@ -536,6 +574,16 @@ const command_rec passenger_commands[] = {
 		NULL,
 		OR_OPTIONS,
 		"The directory in which upload buffer files should be placed."),
+	AP_INIT_TAKE1("UnionStationKey",
+		(Take1Func) cmd_union_station_key,
+		NULL,
+		OR_ALL,
+		"The Union Station key."),
+	AP_INIT_TAKE1("UnionStationFilter",
+		(Take1Func) cmd_union_station_filter,
+		NULL,
+		OR_ALL,
+		"A filter for Union Station data."),
 	AP_INIT_FLAG("PassengerResolveSymlinksInDocumentRoot",
 		(FlagFunc) cmd_passenger_resolve_symlinks_in_document_root,
 		NULL,
@@ -556,6 +604,21 @@ const command_rec passenger_commands[] = {
 		NULL,
 		RSRC_CONF,
 		"The spawn method to use."),
+	AP_INIT_TAKE1("UnionStationGatewayAddress",
+		(Take1Func) cmd_union_station_gateway_address,
+		NULL,
+		RSRC_CONF,
+		"The Union Station Gateway host name."),
+	AP_INIT_TAKE1("UnionStationGatewayPort",
+		(Take1Func) cmd_union_station_gateway_port,
+		NULL,
+		RSRC_CONF,
+		"The Union Station Gateway port number."),
+	AP_INIT_TAKE1("UnionStationGatewayCert",
+		(Take1Func) cmd_union_station_gateway_cert,
+		NULL,
+		RSRC_CONF,
+		"The Union Station Gateway certificate."),
 	AP_INIT_TAKE1("PassengerAnalyticsLogDir",
 		(Take1Func) cmd_passenger_analytics_log_dir,
 		NULL,
@@ -581,11 +644,11 @@ const command_rec passenger_commands[] = {
 		NULL,
 		RSRC_CONF,
 		"Prestart the given web applications during startup."),
-	AP_INIT_TAKE1("PassengerAnalytics",
-		(Take1Func) cmd_passenger_analytics,
+	AP_INIT_FLAG("UnionStationSupport",
+		(Take1Func) cmd_union_station_support,
 		NULL,
 		OR_OPTIONS | ACCESS_CONF | RSRC_CONF,
-		"Whether to enable analytics logging."),
+		"Whether to enable logging through Union Station."),
 	
 	/*****************************/
 
@@ -601,7 +664,7 @@ const command_rec passenger_commands[] = {
 		RSRC_CONF,
 		"Whether auto-detection of Ruby on Rails applications should be enabled."),
 	AP_INIT_TAKE1("RailsEnv",
-		(Take1Func) cmd_rails_env,
+		(Take1Func) cmd_environment,
 		NULL,
 		OR_OPTIONS | ACCESS_CONF | RSRC_CONF,
 		"The environment under which a Rails app must run."),
@@ -628,7 +691,7 @@ const command_rec passenger_commands[] = {
 		RSRC_CONF,
 		"Whether auto-detection of Rack applications should be enabled."),
 	AP_INIT_TAKE1("RackEnv",
-		(Take1Func) cmd_rack_env,
+		(Take1Func) cmd_environment,
 		NULL,
 		OR_OPTIONS | ACCESS_CONF | RSRC_CONF,
 		"The environment under which a Rack app must run."),
