@@ -236,7 +236,8 @@ private:
 	 */
 	inline RequestNote *getRequestNote(request_rec *r) {
 		// The union is needed in order to be compliant with
-		// C99/C++'s strict aliasing rules. http://tinyurl.com/g5hgh
+		// C99/C++'s strict aliasing rules.
+		// http://cellperformance.beyond3d.com/articles/2006/06/understanding-strict-aliasing.html
 		union {
 			RequestNote *note;
 			void *pointer;
@@ -565,8 +566,12 @@ private:
 		UPDATE_TRACE_POINT();
 		try {
 			AnalyticsLogPtr log;
-			if (config->analyticsEnabled()) {
-				log = analyticsLogger->newTransaction(config->getAppGroupName(appRoot));
+			if (config->useUnionStation()) {
+				log = analyticsLogger->newTransaction(
+					config->getAppGroupName(appRoot),
+					"requests",
+					config->unionStationKey,
+					config->getUnionStationFilterString());
 				log->message(string("URI: ") + r->uri);
 			} else {
 				log.reset(new AnalyticsLog());
@@ -640,7 +645,7 @@ private:
 					appRoot,
 					config->getAppGroupName(appRoot),
 					mapper.getApplicationTypeString(),
-					mapper.getEnvironment(),
+					config->getEnvironment(),
 					config->getSpawnMethodString(),
 					config->getUser(),
 					config->getGroup(),
@@ -657,7 +662,7 @@ private:
 					config->getRestartDir(),
 					DEFAULT_BACKEND_ACCOUNT_RIGHTS,
 					false,
-					config->analyticsEnabled(),
+					config->useUnionStation(),
 					log->isNull() ? AnalyticsLogPtr() : log
 				);
 				options.environmentVariables = ptr(new EnvironmentVariablesStringListCreator(r));
@@ -765,6 +770,7 @@ private:
 						r->status);
 				}
 				apr_table_setn(r->headers_out, "Status", r->status_line);
+				log->message(string("Status: ") + r->status_line);
 				
 				bool xsendfile = hasModXsendfile() &&
 					apr_table_get(r->err_headers_out, "X-Sendfile");
@@ -870,46 +876,92 @@ private:
 		}
 	}
 	
+	unsigned int
+	escapeUri(unsigned char *dst, const unsigned char *src, size_t size) {
+		static const char hex[] = "0123456789abcdef";
+			           /* " ", "#", "%", "?", %00-%1F, %7F-%FF */
+		static uint32_t escape[] = {
+		       0xffffffff, /* 1111 1111 1111 1111  1111 1111 1111 1111 */
+
+		                   /* ?>=< ;:98 7654 3210  /.-, +*)( '&%$ #"!  */
+		       0x80000029, /* 1000 0000 0000 0000  0000 0000 0010 1001 */
+
+		                   /* _^]\ [ZYX WVUT SRQP  ONML KJIH GFED CBA@ */
+		       0x00000000, /* 0000 0000 0000 0000  0000 0000 0000 0000 */
+
+		                   /*  ~}| {zyx wvut srqp  onml kjih gfed cba` */
+		       0x80000000, /* 1000 0000 0000 0000  0000 0000 0000 0000 */
+
+		       0xffffffff, /* 1111 1111 1111 1111  1111 1111 1111 1111 */
+		       0xffffffff, /* 1111 1111 1111 1111  1111 1111 1111 1111 */
+		       0xffffffff, /* 1111 1111 1111 1111  1111 1111 1111 1111 */
+		       0xffffffff  /* 1111 1111 1111 1111  1111 1111 1111 1111 */
+		};
+		
+		if (dst == NULL) {
+			/* find the number of the characters to be escaped */
+			unsigned int n = 0;
+			while (size > 0) {
+				if (escape[*src >> 5] & (1 << (*src & 0x1f))) {
+					n++;
+				}
+				src++;
+				size--;
+			}
+			return n;
+		}
+		
+		while (size > 0) {
+			if (escape[*src >> 5] & (1 << (*src & 0x1f))) {
+				*dst++ = '%';
+				*dst++ = hex[*src >> 4];
+				*dst++ = hex[*src & 0xf];
+				src++;
+			} else {
+				*dst++ = *src++;
+			}
+			size--;
+		}
+		return 0;
+	}
+	
 	/**
 	 * Convert an HTTP header name to a CGI environment name.
 	 */
-	char *http2env(apr_pool_t *p, const char *name) {
-		char *env_name = apr_pstrcat(p, "HTTP_", name, NULL);
-		char *cp;
+	char *httpToEnv(apr_pool_t *p, const char *headerName) {
+		char *result  = apr_pstrcat(p, "HTTP_", headerName, NULL);
+		char *current = result + sizeof("HTTP_") - 1;
 		
-		for (cp = env_name + 5; *cp != 0; cp++) {
-			if (*cp == '-') {
-				*cp = '_';
+		while (*current != '\0') {
+			if (*current == '-') {
+				*current = '_';
 			} else {
-				*cp = apr_toupper(*cp);
+				*current = apr_toupper(*current);
 			}
+			current++;
 		}
 		
-		return env_name;
+		return result;
 	}
 	
-	char *lookupName(apr_table_t *t, const char *name) {
-		const apr_array_header_t *hdrs_arr = apr_table_elts(t);
-		apr_table_entry_t *hdrs = (apr_table_entry_t *) hdrs_arr->elts;
-		int i;
+	const char *lookupInTable(apr_table_t *table, const char *name) {
+		const apr_array_header_t *headers = apr_table_elts(table);
+		apr_table_entry_t *elements = (apr_table_entry_t *) headers->elts;
 		
-		for (i = 0; i < hdrs_arr->nelts; ++i) {
-			if (hdrs[i].key == NULL) {
-				continue;
-			}
-			if (strcasecmp(hdrs[i].key, name) == 0) {
-				return hdrs[i].val;
+		for (int i = 0; i < headers->nelts; i++) {
+			if (elements[i].key != NULL && strcasecmp(elements[i].key, name) == 0) {
+				return elements[i].val;
 			}
 		}
 		return NULL;
 	}
 	
-	char *lookupHeader(request_rec *r, const char *name) {
-		return lookupName(r->headers_in, name);
+	const char *lookupHeader(request_rec *r, const char *name) {
+		return lookupInTable(r->headers_in, name);
 	}
 	
-	char *lookupEnv(request_rec *r, const char *name) {
-		return lookupName(r->subprocess_env, name);
+	const char *lookupEnv(request_rec *r, const char *name) {
+		return lookupInTable(r->subprocess_env, name);
 	}
 	
 	void inline addHeader(apr_table_t *table, const char *name, const char *value) {
@@ -927,6 +979,17 @@ private:
 			return APR_ENOMEM;
 		}
 		
+		/*
+		 * Apache unescapes URI's before passing them to Phusion Passenger,
+		 * but backend processes expect the escaped version.
+		 * http://code.google.com/p/phusion-passenger/issues/detail?id=404
+		 */
+		size_t uriLen = strlen(r->uri);
+		unsigned int escaped = escapeUri(NULL, (const unsigned char *) r->uri, uriLen);
+		char escapedUri[uriLen + 2 * escaped + 1];
+		escapeUri((unsigned char *) escapedUri, (const unsigned char *) r->uri, uriLen);
+		escapedUri[uriLen + 2 * escaped] = '\0';
+		
 		
 		// Set standard CGI variables.
 		addHeader(headers, "SERVER_SOFTWARE", ap_get_server_version());
@@ -958,19 +1021,19 @@ private:
 		} else {
 			const char *request_uri;
 			if (r->args != NULL) {
-				request_uri = apr_pstrcat(r->pool, r->uri, "?", r->args, NULL);
+				request_uri = apr_pstrcat(r->pool, escapedUri, "?", r->args, NULL);
 			} else {
-				request_uri = r->uri;
+				request_uri = escapedUri;
 			}
 			addHeader(headers, "REQUEST_URI", request_uri);
 		}
 		
 		if (strcmp(baseURI, "/") == 0) {
 			addHeader(headers, "SCRIPT_NAME", "");
-			addHeader(headers, "PATH_INFO", r->uri);
+			addHeader(headers, "PATH_INFO", escapedUri);
 		} else {
 			addHeader(headers, "SCRIPT_NAME", baseURI);
-			addHeader(headers, "PATH_INFO", r->uri + strlen(baseURI));
+			addHeader(headers, "PATH_INFO", escapedUri + strlen(baseURI));
 		}
 		
 		// Set HTTP headers.
@@ -982,7 +1045,7 @@ private:
 		hdrs = (apr_table_entry_t *) hdrs_arr->elts;
 		for (i = 0; i < hdrs_arr->nelts; ++i) {
 			if (hdrs[i].key) {
-				addHeader(headers, http2env(r->pool, hdrs[i].key), hdrs[i].val);
+				addHeader(headers, httpToEnv(r->pool, hdrs[i].key), hdrs[i].val);
 			}
 		}
 		
@@ -996,10 +1059,11 @@ private:
 			addHeader(headers, env[i].key, env[i].val);
 		}
 		
-		if (config->analyticsEnabled()) {
+		if (!log->isNull()) {
 			addHeader(headers, "PASSENGER_GROUP_NAME",
 				config->getAppGroupName(appRoot).c_str());
 			addHeader(headers, "PASSENGER_TXN_ID", log->getTxnId().c_str());
+			addHeader(headers, "PASSENGER_UNION_STATION_KEY", config->unionStationKey.c_str());
 		}
 		
 		// Now send the headers.
@@ -1337,9 +1401,9 @@ public:
 			"",
 			serverConfig.analyticsLogDir, serverConfig.analyticsLogUser,
 			serverConfig.analyticsLogGroup, serverConfig.analyticsLogPermissions,
-			DEFAULT_UNION_STATION_GATEWAY_ADDRESS,
-			DEFAULT_UNION_STATION_GATEWAY_PORT,
-			"",
+			serverConfig.unionStationGatewayAddress,
+			serverConfig.unionStationGatewayPort,
+			serverConfig.unionStationGatewayCert,
 			serverConfig.prestartURLs);
 		
 		analyticsLogger = ptr(new AnalyticsLogger(agentsStarter.getLoggingSocketAddress(),
@@ -1741,8 +1805,8 @@ passenger_register_hooks(apr_pool_t *p) {
 	ap_hook_fixups(prepare_request_when_not_in_high_performance_mode, NULL, rewrite_module, APR_HOOK_FIRST);
 	ap_hook_fixups(save_state_before_rewrite_rules, NULL, rewrite_module, APR_HOOK_LAST);
 	ap_hook_fixups(undo_redirection_to_dispatch_cgi, rewrite_module, NULL, APR_HOOK_FIRST);
-	ap_hook_fixups(start_blocking_mod_dir, NULL, dir_module, APR_HOOK_MIDDLE);
-	ap_hook_fixups(end_blocking_mod_dir, dir_module, NULL, APR_HOOK_MIDDLE);
+	ap_hook_fixups(start_blocking_mod_dir, NULL, dir_module, APR_HOOK_LAST);
+	ap_hook_fixups(end_blocking_mod_dir, dir_module, NULL, APR_HOOK_LAST);
 	
 	ap_hook_handler(handle_request_when_in_high_performance_mode, NULL, NULL, APR_HOOK_FIRST);
 	ap_hook_handler(start_blocking_mod_autoindex, NULL, autoindex_module, APR_HOOK_LAST);
