@@ -49,6 +49,11 @@
 #include <cerrno>
 #include <cmath>
 
+#ifdef __linux__
+	#include <sys/syscall.h>
+#endif
+
+#include "Timer.h"
 #include "IOUtils.h"
 #include "StrIntUtils.h"
 #include "../Exceptions.h"
@@ -149,6 +154,33 @@ setNonBlocking(int fd) {
 	}
 }
 
+int
+tryAccept4(int sock, struct sockaddr *addr, socklen_t *addr_len, int options) {
+	#if defined(__linux__) && defined(__x86_64__)
+		int ret;
+		do {
+			ret = syscall(288, sock, addr, addr_len, options);
+		} while (ret == -1 && errno == EINTR);
+		return ret;
+	#elif defined(__linux__) && defined(__i386__)
+		int ret;
+		do {
+			ret = syscall(__NR_socketcall, 18,
+				sock, addr, addr_len, options);
+		} while (ret == -1 && errno == EINTR);
+		return ret;
+	#elif defined(SYS_ACCEPT4)
+		int ret;
+		do {
+			ret = ::accept4(sock, addr, addr_len, options);
+		} while (ret == -1 && errno == EINTR);
+		return ret;
+	#else
+		errno = ENOSYS;
+		return -1;
+	#endif
+}
+
 vector<string>
 resolveHostname(const string &hostname, unsigned int port, bool shuffle) {
 	string portString = toString(port);
@@ -504,6 +536,116 @@ connectToTcpServer(const StaticString &hostname, unsigned int port) {
 	return fd;
 }
 
+SocketPair
+createUnixSocketPair() {
+	int fds[2];
+	FileDescriptor sockets[2];
+	
+	if (syscalls::socketpair(AF_UNIX, SOCK_STREAM, 0, fds) == -1) {
+		int e = errno;
+		throw SystemException("Cannot create a Unix socket pair", e);
+	} else {
+		sockets[0] = fds[0];
+		sockets[1] = fds[1];
+		return SocketPair(sockets[0], sockets[1]);
+	}
+}
+
+Pipe
+createPipe() {
+	int fds[2];
+	FileDescriptor p[2];
+	
+	if (syscalls::pipe(fds) == -1) {
+		int e = errno;
+		throw SystemException("Cannot create a pipe", e);
+	} else {
+		p[0] = fds[0];
+		p[1] = fds[1];
+		return Pipe(p[0], p[1]);
+	}
+}
+
+static bool
+waitUntilIOEvent(int fd, short event, unsigned long long *timeout) {
+	struct pollfd pfd;
+	int ret;
+	
+	pfd.fd = fd;
+	pfd.events = event;
+	pfd.revents = 0;
+	
+	Timer timer;
+	ret = syscalls::poll(&pfd, 1, *timeout / 1000);
+	if (ret == -1) {
+		int e = errno;
+		throw SystemException("poll() failed", e);
+	} else {
+		unsigned long long elapsed = timer.usecElapsed();
+		if (elapsed > *timeout) {
+			*timeout = 0;
+		} else {
+			*timeout -= elapsed;
+		}
+		return ret != 0;
+	}
+}
+
+bool
+waitUntilReadable(int fd, unsigned long long *timeout) {
+	return waitUntilIOEvent(fd, POLLIN, timeout);
+}
+
+bool
+waitUntilWritable(int fd, unsigned long long *timeout) {
+	return waitUntilIOEvent(fd, POLLOUT | POLLHUP, timeout);
+}
+
+unsigned int
+readExact(int fd, void *buf, unsigned int size, unsigned long long *timeout) {
+	ssize_t ret;
+	unsigned int alreadyRead = 0;
+	
+	while (alreadyRead < size) {
+		if (timeout != NULL && !waitUntilReadable(fd, timeout)) {
+			throw TimeoutException("Cannot read enough data within the specified timeout");
+		}
+		ret = syscalls::read(fd, (char *) buf + alreadyRead, size - alreadyRead);
+		if (ret == -1) {
+			int e = errno;
+			throw SystemException("read() failed", e);
+		} else if (ret == 0) {
+			return alreadyRead;
+		} else {
+			alreadyRead += ret;
+		}
+	}
+	return alreadyRead;
+}
+
+void
+writeExact(int fd, const void *data, unsigned int size, unsigned long long *timeout) {
+	ssize_t ret;
+	unsigned int written = 0;
+	while (written < size) {
+		if (timeout != NULL && !waitUntilWritable(fd, timeout)) {
+			throw TimeoutException("Cannot write enough data within the specified timeout");
+		}
+		ret = syscalls::write(fd, (const char *) data + written, size - written);
+		if (ret == -1) {
+			int e = errno;
+			throw SystemException("write() failed", e);
+		} else {
+			written += ret;
+		}
+	}
+}
+
+void
+writeExact(int fd, const StaticString &data, unsigned long long *timeout) {
+	writeExact(fd, data.c_str(), data.size(), timeout);
+}
+
 /**
  * Converts an array of StaticStrings to a corresponding array of iovec structures,
  * returning the size sum in bytes of all StaticStrings.
@@ -750,5 +892,22 @@ setWritevFunction(WritevFunction func) {
 	}
 }
 
+void
+safelyClose(int fd) {
+	if (syscalls::close(fd) == -1) {
+		/* FreeBSD has a kernel bug which can cause close() to return ENOTCONN.
+		 * This is harmless, ignore it. We check for this problem on all
+		 * platforms because some OSes might borrow Unix domain socket
+		 * code from FreeBSD.
+		 * http://www.freebsd.org/cgi/query-pr.cgi?pr=79138
+		 * http://www.freebsd.org/cgi/query-pr.cgi?pr=144061
+		 */
+		if (errno != ENOTCONN) {
+			int e = errno;
+			throw SystemException("Cannot close file descriptor", e);
+		}
+	}
+}
+
 
 } // namespace Passenger
