@@ -313,12 +313,13 @@ private:
 	 *                                     before we were able to send back the
 	 *                                     full response.
 	 */
-	void forwardResponse(SessionPtr &session, FileDescriptor &clientFd) {
+	void forwardResponse(SessionPtr &session, FileDescriptor &clientFd,
+		const AnalyticsLogPtr &log)
+	{
 		TRACE_POINT();
 		HttpStatusExtractor ex;
 		int stream = session->getStream();
 		int eof = false;
-		MessageChannel output(clientFd);
 		char buf[1024 * 32];
 		ssize_t size;
 		
@@ -343,9 +344,14 @@ private:
 					string statusLine("HTTP/1.1 ");
 					statusLine.append(ex.getStatusLine());
 					UPDATE_TRACE_POINT();
-					output.writeRaw(statusLine.c_str(), statusLine.size());
+					writeExact(clientFd, statusLine);
 					UPDATE_TRACE_POINT();
-					output.writeRaw(ex.getBuffer().c_str(), ex.getBuffer().size());
+					writeExact(clientFd, ex.getBuffer());
+					if (!log->isNull()) {
+						string partialStatusLine = ex.getStatusLine();
+						partialStatusLine.erase(partialStatusLine.size() - 2, 2);
+						log->message("Status: " + partialStatusLine);
+					}
 					break;
 				} catch (const SystemException &e) {
 					if (e.code() == EPIPE) {
@@ -368,7 +374,7 @@ private:
 			} else {
 				UPDATE_TRACE_POINT();
 				try {
-					output.writeRaw(buf, size);
+					writeExact(clientFd, buf, size);
 				} catch (const SystemException &e) {
 					if (e.code() == EPIPE) {
 						throw ClientDisconnectedException();
@@ -392,31 +398,30 @@ private:
 	 * @param friendly Whether to show a friendly error page.
 	 */
 	void handleSpawnException(FileDescriptor &fd, const SpawnException &e, bool friendly) {
-		MessageChannel channel(fd);
-		channel.writeRaw("HTTP/1.1 500 Internal Server Error\x0D\x0A");
-		channel.writeRaw("Status: 500 Internal Server Error\x0D\x0A");
-		channel.writeRaw("Connection: close\x0D\x0A");
-		channel.writeRaw("Content-Type: text/html; charset=utf-8\x0D\x0A");
+		writeExact(fd, "HTTP/1.1 500 Internal Server Error\x0D\x0A");
+		writeExact(fd, "Status: 500 Internal Server Error\x0D\x0A");
+		writeExact(fd, "Connection: close\x0D\x0A");
+		writeExact(fd, "Content-Type: text/html; charset=utf-8\x0D\x0A");
 		
 		if (friendly) {
 			if (e.hasErrorPage()) {
-				channel.writeRaw("Content-Length: " +
+				writeExact(fd, "Content-Length: " +
 					toString(e.getErrorPage().size()) +
 					"\x0D\x0A");
-				channel.writeRaw("\x0D\x0A");
-				channel.writeRaw(e.getErrorPage());
+				writeExact(fd, "\x0D\x0A");
+				writeExact(fd, e.getErrorPage());
 			} else {
-				channel.writeRaw("Content-Length: " +
+				writeExact(fd, "Content-Length: " +
 					toString(strlen(e.what())) + "\x0D\x0A");
-				channel.writeRaw("\x0D\x0A");
-				channel.writeRaw(e.what());
+				writeExact(fd, "\x0D\x0A");
+				writeExact(fd, e.what());
 			}
 		} else {
 			const char body[] = "<h1>Internal Server Error (500)</h1>";
-			channel.writeRaw("Content-Length: " +
+			writeExact(fd, "Content-Length: " +
 				toString(strlen(body)) + "\x0D\x0A");
-			channel.writeRaw("\x0D\x0A");
-			channel.writeRaw(body);
+			writeExact(fd, "\x0D\x0A");
+			writeExact(fd, body);
 		}
 	}
 	
@@ -439,7 +444,7 @@ private:
 		}
 		
 		try {
-			bool enableAnalytics = parser.getHeader("PASSENGER_ANALYTICS") == "true";
+			bool useUnionStation = parser.getHeader("UNION_STATION_SUPPORT") == "true";
 			StaticString appGroupName = parser.getHeader("PASSENGER_APP_GROUP_NAME");
 			PoolOptions options;
 			
@@ -473,11 +478,12 @@ private:
 			
 			UPDATE_TRACE_POINT();
 			AnalyticsLogPtr log;
-			if (enableAnalytics) {
+			if (useUnionStation) {
 				log = analyticsLogger->newTransaction(
 					options.getAppGroupName(),
 					"requests",
-					parser.getHeader("PASSENGER_UNION_STATION_KEY"));
+					parser.getHeader("PASSENGER_UNION_STATION_KEY"),
+					parser.getHeader("UNION_STATION_FILTERS"));
 				options.analytics = true;
 				options.log = log;
 			} else {
@@ -523,7 +529,7 @@ private:
 					session->getConnectPassword().size() + 1);
 				end += session->getConnectPassword().size() + 1;
 				
-				if (enableAnalytics) {
+				if (!log->isNull()) {
 					memcpy(end, "PASSENGER_GROUP_NAME", sizeof("PASSENGER_GROUP_NAME"));
 					end += sizeof("PASSENGER_GROUP_NAME");
 					
@@ -556,7 +562,7 @@ private:
 					scope.success();
 				}
 				
-				forwardResponse(session, clientFd);
+				forwardResponse(session, clientFd, log);
 				
 				requestProxyingScope.success();
 			} catch (const SpawnException &e) {
@@ -607,10 +613,6 @@ private:
 				<< "   exception: " << e.what() << "\n"
 				<< "   backtrace:\n" << e.backtrace());
 			abort();
-		} catch (const std::exception &e) {
-			P_ERROR("Uncaught exception in PassengerServer client thread:\n"
-				<< "   exception: " << e.what() << "\n"
-				<< "   backtrace: not available");
 		}
 	}
 	
@@ -639,7 +641,7 @@ public:
 		this->serverSocket = serverSocket;
 		this->analyticsLogger = logger;
 		thr = new oxt::thread(
-			bind(&Client::threadMain, this),
+			boost::bind(&Client::threadMain, this),
 			"Client thread " + toString(number),
 			CLIENT_THREAD_STACK_SIZE
 		);
@@ -865,8 +867,11 @@ public:
 			messageServer->getSocketFilename().c_str(),
 			NULL);
 		
+		function<void ()> func = boost::bind(prestartWebApps,
+			resourceLocator,
+			options.get("prestart_urls"));
 		prestarterThread = ptr(new oxt::thread(
-			boost::bind(prestartWebApps, resourceLocator, options.get("prestart_urls"))
+			boost::bind(runAndPrintExceptions, func, true)
 		));
 	}
 	
@@ -898,8 +903,9 @@ public:
 		TRACE_POINT();
 		
 		startClientHandlerThreads();
+		function<void ()> func = boost::bind(&MessageServer::mainLoop, messageServer.get());
 		messageServerThread = ptr(new oxt::thread(
-			boost::bind(&MessageServer::mainLoop, messageServer.get()),
+			boost::bind(runAndPrintExceptions, func, true),
 			"MessageServer thread", MESSAGE_SERVER_THREAD_STACK_SIZE
 		));
 		
